/*
  Compile with 
    gcc -Wall -D__USE_INLINE__ getstr.c -o getstr -lauto
  or
    vc +aosppc -+ -D__USE_INLINE__ getstr.c -o getstr -lauto
  or
    vc +aos68k -+ getstr.c -o getstr -lamiga -lauto
*/

/*----------------------------------------------------------------------------*/
/* System includes                                                            */
/*----------------------------------------------------------------------------*/

	#include <clib/alib_protos.h>
	#include <proto/exec.h>
	#include <proto/dos.h>
	#include <proto/graphics.h>
	#include <proto/intuition.h>
	#include <proto/utility.h>
	#include <workbench/startup.h>
	#include <workbench/workbench.h>

	#define ALL_REACTION_CLASSES
	#include <reaction/reaction.h>
	#include <reaction/reaction_macros.h>

/*----------------------------------------------------------------------------*/
/* Constants and macros                                                       */
/*----------------------------------------------------------------------------*/

#ifdef __amigaos4__
	#define BASEDEF(base) struct Library *base##Base = NULL; struct base##IFace *I##base = NULL
	#define OPENLIB(base,name) if (!(base##Base = OpenLibrary (name,0)) || !(I##base = (struct base##IFace *)GetInterface ((struct Library *)base##Base,"main",1,NULL))) ok = FALSE
	#define CLOSELIB(base) if (I##base) DropInterface ((struct Interface *)I##base); if (base##Base) CloseLibrary (base##Base)
#else
	#define BASEDEF(base) struct Library *base##Base = NULL
	#define OPENLIB(base,name) if (!(base##Base = OpenLibrary (name,0))) ok = FALSE
	#define CLOSELIB(base) if (base##Base) CloseLibrary (base##Base)
#endif

/*----------------------------------------------------------------------------*/
/* Type definitions                                                           */
/*----------------------------------------------------------------------------*/

	typedef union {
		ULONG pens;
		struct {
			UWORD back;
			UWORD text;
			} pen;
		} PENS;
/*----------------------------------------------------------------------------*/
/* Global variables                                                           */
/*----------------------------------------------------------------------------*/

	BASEDEF(Window);
	BASEDEF(Layout);
	BASEDEF(Button);
	BASEDEF(String);

/*----------------------------------------------------------------------------*/
/* Open required class libs                                                   */
/*----------------------------------------------------------------------------*/

	BOOL open_libs (void)

	{
	BOOL ok = TRUE;

	OPENLIB(Window,"window.class");
	OPENLIB(Layout,"gadgets/layout.gadget");
	OPENLIB(Button,"gadgets/button.gadget");
	OPENLIB(String,"gadgets/string.gadget");

	return (ok);
	}

/*----------------------------------------------------------------------------*/
/* Close class libs                                                           */
/*----------------------------------------------------------------------------*/

	void close_libs (void)

	{
	CLOSELIB(Window);
	CLOSELIB(Layout);
	CLOSELIB(Button);
	CLOSELIB(String);
	}

/*----------------------------------------------------------------------------*/
/* Main program                                                               */
/*----------------------------------------------------------------------------*/

	int main (void)

	{
	struct Screen *scr;

	if (!open_libs())
		{
		Printf ("cannot open required gui classes\n");
		close_libs();
		return (20);
		}

	if (scr = LockPubScreen (NULL))
		{
		Object *winobj;
		PENS normal;
		PENS active;

		normal.pen.back = ObtainBestPen (scr->ViewPort.ColorMap,0x00000000,0x00000000,0xffffffff,OBP_FailIfBad,FALSE,OBP_Precision,PRECISION_GUI,TAG_END);
		normal.pen.text = ObtainBestPen (scr->ViewPort.ColorMap,0x00000000,0x88888888,0xffffffff,OBP_FailIfBad,FALSE,OBP_Precision,PRECISION_GUI,TAG_END);
		active.pen.back = ObtainBestPen (scr->ViewPort.ColorMap,0x44444444,0xaaaaaaaa,0xffffffff,OBP_FailIfBad,FALSE,OBP_Precision,PRECISION_GUI,TAG_END);
		active.pen.text = ObtainBestPen (scr->ViewPort.ColorMap,0xffffffff,0xffffffff,0xffffffff,OBP_FailIfBad,FALSE,OBP_Precision,PRECISION_GUI,TAG_END);

		if ((winobj = WindowObject,
				WA_PubScreen,scr,
				WA_Flags,WFLG_CLOSEGADGET | WFLG_DRAGBAR | WFLG_DEPTHGADGET | WFLG_SIZEBBOTTOM | WFLG_SIZEGADGET | WFLG_ACTIVATE,
				WA_IDCMP,IDCMP_VANILLAKEY,
				WA_Title,"String",
				WINDOW_Position,WPOS_CENTERSCREEN,
				WINDOW_ParentGroup,VGroupObject,
					LAYOUT_SpaceOuter,TRUE,
					LAYOUT_AddChild,StringObject,
						STRINGA_MaxChars,80,
						STRINGA_Pens,normal.pens,
						STRINGA_ActivePens,active.pens,
						STRINGA_TextVal,"Hello World!",
					EndObject,
					StartHGroup,
						LAYOUT_FixedVert,FALSE,
						LAYOUT_EvenSize,TRUE,
						LAYOUT_AddChild,
							Button("Ok",1),
						LAYOUT_AddChild,
							Button("Cancel",2),
					EndHGroup,
				EndVGroup,
			EndWindow))

			{
			struct Window *win;

			if ((win = RA_OpenWindow (winobj)))
				{
				BOOL cont = TRUE;
				ULONG winsig = 0;

				GetAttr (WINDOW_SigMask, winobj, &winsig);

				do	{
					ULONG sigs = Wait (winsig | SIGBREAKF_CTRL_C);

					if (sigs & SIGBREAKF_CTRL_C)
						cont = FALSE;

					if (sigs & winsig)
						{
						ULONG result;
						while ((result = RA_HandleInput ((Object *)winobj, NULL)) != WMHI_LASTMSG)
							{
							switch (result & WMHI_CLASSMASK)
								{
							case WMHI_GADGETUP:
								switch (result & WMHI_GADGETMASK)
									{
								case 1:
								case 2:
									cont = FALSE;
									break;
									}
								break;
							case WMHI_VANILLAKEY:
								if ((result & WMHI_KEYMASK) == 0x1b)
									cont = FALSE;
								break;
							case WMHI_CLOSEWINDOW:
								cont = FALSE;
								break;
								}
							}
						}
					}
				while (cont);
				}

			DisposeObject (winobj);
			}

		ReleasePen (scr->ViewPort.ColorMap,normal.pen.back);
		ReleasePen (scr->ViewPort.ColorMap,normal.pen.text);
		ReleasePen (scr->ViewPort.ColorMap,active.pen.back);
		ReleasePen (scr->ViewPort.ColorMap,active.pen.text);

		UnlockPubScreen (NULL,scr);
		}

	close_libs();

	return (0);
	}

/*----------------------------------------------------------------------------*/
/* Ende des Quelltextes                                                       */
/*----------------------------------------------------------------------------*/
